/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2008-2012, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#include <arpa/inet.h>
#include "users.h"
#include "sessionmanager.h"
#include "plugin.h"



Users::Users()
{
	how_many_logged = 0; // !! CHECK ME may it should be moved to Clear() method?
	Clear();
}


void Users::SetCur(Cur * pcur)
{
	cur = pcur;
}


void Users::SetSessionManager(SessionManager * sm)
{
	session_manager = sm;
}


void Users::Clear()
{
	table.Clear();	
}



void Users::ReadUsers(Db * db)
{
	Clear();	
	db->GetUsers(table);
}


void Users::SetTimeZoneOffset(int offset)
{
	Table::Iterator i;

	for(i=table.Begin() ; i!=table.End() ; ++i)
	{
		i->time_zone_offset = offset;
	}
}


bool Users::AddUser(const User & user)
{
	Table::Iterator i = table.PushBack(user);
	
return (i != table.End());
}


bool Users::IsUser(const std::wstring & name)
{
	return table.Is(name);
}


User * Users::GetUser(long user_id)
{
	Table::Iterator i = table.FindId(user_id);
	
	if( i == table.End() )
		return 0;
	
return &(*i);
}


User * Users::GetUser(const std::wstring & name)
{
	Table::Iterator i = table.FindName(name);
	
	if( i == table.End() )
		return 0;
	
return &(*i);
}



long Users::GetUserId(const std::wstring & name)
{
User * puser = GetUser(name);

	if( !puser )
		return -1;	
	
return puser->id;
}



Users::Iterator Users::Begin()
{
	return table.Begin();
}



Users::Iterator Users::End()
{
	return table.End();
}



Users::SizeType Users::Size()
{
	return table.Size();
}


bool Users::Remove(long user_id)
{
	bool result = false;
	User * puser = GetUser(user_id);

	if( puser )
	{
		LogoutUser(user_id);
		plugin.Call(WINIX_PREPARE_TO_REMOVE_USER, puser);
		result = table.Remove(user_id);

		if( result )
			plugin.Call(WINIX_USER_REMOVED, user_id);
	}

return result;
}


// private
bool Users::LoginUserCheckSession(bool use_ses_log)
{
	if( !cur->session )
		return false;

	if( cur->session->id == 0 )
	{
		log << log1 << "Users: I cannot login a user on a temporary session" << logend;

		if( use_ses_log )
			slog << logerror << T(L"service_unavailable") << logend;

		return false;
	}

return true;
}


// private
User * Users::LoginUserCheckStatus(long user_id, bool use_ses_log)
{
	User * puser = GetUser(user_id);

	if( !puser )
	{
		log << log1 << "Users: user id: " << user_id << " is not in system.users table" << logend;

		if( use_ses_log )
			slog << logerror << T(L"service_unavailable") << logend;

		return 0;
	}

	if( puser->status != WINIX_ACCOUNT_READY )
	{
		log << log1 << "Users: user id: " << user_id << " is not ready for logging in (status: "
			<< puser->status << ")" << logend;

		if( use_ses_log )
		{
			if( puser->status == WINIX_ACCOUNT_NOT_ACTIVATED )
				slog << logerror << T(L"account_not_activated") << logend;
			else
			if( puser->status == WINIX_ACCOUNT_SUSPENDED )
				slog << logerror << T(L"account_suspended") << logend;
			else
			if( puser->status == WINIX_ACCOUNT_BLOCKED )
				slog << logerror << T(L"account_banned") << logend;
		}

		return 0;
	}

return puser;
}



bool Users::LoginUser(long user_id, bool remember_me, bool use_ses_log)
{
	if( !LoginUserCheckSession(use_ses_log) )
		return false;

	User * puser = LoginUserCheckStatus(user_id, use_ses_log);

	if( !puser )
		return false;

	plugin.Call(WINIX_PREPARE_USER_TO_LOGIN, puser);

	if( plugin.False() > 0 )
	{
		log << log3 << "Users: login prevented by a plugin" << logend;
		return false;
	}

	if( cur->session->puser )
		LogoutCurrentUser();

	cur->session->puser       = puser;
	cur->session->spam_score  = 0;
	cur->session->remember_me = remember_me;

	// change session id before last.UserLogin()
	if( !cur->session->new_session )
		session_manager->ChangeSessionId(cur->session->id);

	last.UserLogin(user_id, cur->session->puser->name, inet_addr(cur->request->env_remote_addr), cur->session->id);
	how_many_logged += 1;

	log << log2 << "Users: user " << cur->session->puser->name << " (id: " << user_id << ") logged" << logend;
	plugin.Call(WINIX_USER_LOGGED);

return true;
}


size_t Users::LogoutUser(long user_id)
{
	size_t how_many = 0;
	User * puser = GetUser(user_id);

	if( puser )
	{
		log << log2 << "Users: logging out user " << puser->name <<  ", id: "
			<< puser->id << " from all sessions" << logend;
		
		// WINIX_PREPARE_USER_TO_LOGOUT will be sent by MarkAllSessionsToRemove()
		how_many = session_manager->MarkAllSessionsToRemove(user_id);
		how_many_logged -= how_many;

		if( how_many )
			log << log3 << "Users: " << how_many << " user(s) were logged out" << logend;
	}

return how_many;
}



void Users::LogoutCurrentUser()
{
	if( !cur->session || !cur->session->puser )
		return;

	log << log2 << "Users: user " << cur->session->puser->name <<  ", id: "
		<< cur->session->puser->id << " logged out" << logend;
	
	plugin.Call(WINIX_PREPARE_USER_TO_LOGOUT, cur->session->puser);
	last.UserLogout(cur->session->puser->id, cur->session->id);

	if( how_many_logged > 0 ) // for safety
		how_many_logged -= 1;
	
	cur->session->puser = 0;
	cur->session->remember_me = false;
}


void Users::IncrementLoggedUsers()
{
	how_many_logged += 1;
}


long Users::HowManyLogged()
{
	return how_many_logged;
}




